/*
 * Copyright (c) 2017-2019 Arm Limited.
 *
 * SPDX-License-Identifier: MIT
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
#ifndef ARM_COMPUTE_GCABSOLUTEDIFFERENCEKERNEL_H
#define ARM_COMPUTE_GCABSOLUTEDIFFERENCEKERNEL_H

#include "arm_compute/core/GLES_COMPUTE/IGCKernel.h"

namespace arm_compute
{
class IGCTensor;

/** Interface for the absolute difference kernel.
 *
 * Absolute difference is computed by:
 * @f[ output(x,y) = | input1(x,y) - input2(x,y) | @f]
 */
class GCAbsoluteDifferenceKernel : public IGCKernel
{
public:
    /** Default constructor. */
    GCAbsoluteDifferenceKernel();
    /** Prevent instances of this class from being copied (As this class contains pointers) */
    GCAbsoluteDifferenceKernel(const GCAbsoluteDifferenceKernel &) = delete;
    /** Prevent instances of this class from being copied (As this class contains pointers) */
    GCAbsoluteDifferenceKernel &operator=(const GCAbsoluteDifferenceKernel &) = delete;
    /** Allow instances of this class to be moved */
    GCAbsoluteDifferenceKernel(GCAbsoluteDifferenceKernel &&) = default;
    /** Allow instances of this class to be moved */
    GCAbsoluteDifferenceKernel &operator=(GCAbsoluteDifferenceKernel &&) = default;
    /** Default destructor */
    ~GCAbsoluteDifferenceKernel() = default;

    /** Set the inputs and output images.
     *
     * @param[in]  input1 Source tensor. Data types supported: U8
     * @param[in]  input2 Source tensor. Data types supported: U8
     * @param[out] output Destination tensor. Data types supported: U8
     */
    void configure(const IGCTensor *input1, const IGCTensor *input2, IGCTensor *output);

    // Inherited methods overridden:
    void run(const Window &window) override;

private:
    const IGCTensor *_input1; /**< Source tensor 1. */
    const IGCTensor *_input2; /**< Source tensor 2. */
    IGCTensor       *_output; /**< Destination tensor. */
};
}
#endif /* ARM_COMPUTE_GCABSOLUTEDIFFERENCEKERNEL_H */
