/*******************************************************************************
* Copyright 2019-2020 Intel Corporation
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*     http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*******************************************************************************/

#ifndef ONEAPI_DNNL_DNNL_VERSION_H
#define ONEAPI_DNNL_DNNL_VERSION_H

// clang-format off

/// Major version
#define DNNL_VERSION_MAJOR 2

/// Minor version
#define DNNL_VERSION_MINOR 2

/// Patch version
#define DNNL_VERSION_PATCH 0

/// Git commit hash
#define DNNL_VERSION_HASH  "269680b228218158fc172e9d5277446f73ac1917"

// clang-format on

#endif
